@@ -33,7 +33,6 @@
 
 #ifdef HAVE_LANGINFO_YESEXPR
 #include <langinfo.h>
-#include <regex.h>
 #endif
 
 /* not possible to unget more than one char under some curses libs, and it
@@ -265,7 +264,7 @@ void mutt_curses_error (const char *fmt, ...)
   set_option (OPTMSGERR);
 }
 
-void mutt_message (const char *fmt, ...)
+void mutt_curses_message (const char *fmt, ...)
 {
   va_list ap;
 
