@@ -1067,6 +1067,13 @@ struct option_t MuttVars[] = {
   ** \fIpgp-menu\fP, when encryption is not required or signing is
   ** requested as well.
   */
+  { "pgp_ignore_subkeys", DT_BOOL, R_NONE, OPTPGPIGNORESUB, 1},
+  /*
+  ** .pp
+  ** Setting this variable will cause Mutt to ignore OpenPGP subkeys. Instead,
+  ** the principal key will inherit the subkeys' capabilities.  Unset this
+  ** if you want to play interesting key selection games.
+  */
   { "pgp_entry_format", DT_STR,  R_NONE, UL &PgpEntryFormat, UL "%4n %t%f %4l/0x%k %-4a %2c %u" },
   /*
   ** .pp
@@ -1086,7 +1093,7 @@ struct option_t MuttVars[] = {
   ** .dt %[<s>] .dd date of the key where <s> is an strftime(3) expression
   ** .de
   */
-  { "pgp_good_sign",	DT_RX,  R_NONE, UL &PgpGoodSign, UL "" },
+  { "pgp_good_sign",	DT_RX,  R_NONE, UL &PgpGoodSign, 0 },
   /*
   ** .pp
   ** If you assign a text to this variable, then a PGP signature is only
@@ -1518,6 +1525,19 @@ struct option_t MuttVars[] = {
   ** some advanced printer filter which is able to properly format
   ** e-mail messages for printing.
   */
+  { "print_split",	DT_BOOL, R_NONE, OPTPRINTSPLIT,  0 },
+  /*
+  ** .pp
+  ** Used in connection with the print-message command.  If this option
+  ** is set, the command sepcified by $$print_command is executed once for
+  ** each message which is to be printed.  If this option is unset, 
+  ** the command specified by $$print_command is executed only once, and
+  ** all the messages are concatenated, with a form feed as the message
+  ** separator.
+  ** .pp
+  ** Those who use the \fBenscript\fP(1) program's mail-printing mode will
+  ** most likely want to set this option.
+  */
   { "prompt_after",	DT_BOOL, R_NONE, OPTPROMPTAFTER, 1 },
   /*
   ** .pp
@@ -1802,6 +1822,14 @@ struct option_t MuttVars[] = {
   ** detect your signature.  For example, Mutt has the ability to highlight
   ** the signature in a different color in the builtin pager.
   */
+  { "sig_on_top",	DT_BOOL, R_NONE, OPTSIGONTOP, 0},
+  /*
+  ** .pp
+  ** If set, the signature will be included before any quoted or forwarded
+  ** text.  It is \fBstrongly\fP recommended that you do not set this variable
+  ** unless you really know what you are doing, and are prepared to take
+  ** some heat from netiquette guardians.
+  */
   { "signature",	DT_PATH, R_NONE, UL &Signature, UL "~/.signature" },
   /*
   ** .pp
