@@ -339,6 +339,7 @@ enum
   OPTPIPESPLIT,
   OPTPOPLAST,
   OPTPRINTDECODE,
+  OPTPRINTSPLIT,
   OPTPROMPTAFTER,
   OPTREADONLY,
   OPTREPLYSELF,
@@ -352,6 +353,7 @@ enum
   OPTSAVENAME,
   OPTSCORE,
   OPTSIGDASHES,
+  OPTSIGONTOP,
   OPTSORTRE,
   OPTSTATUSONTOP,
   OPTSTRICTTHREADS,
@@ -374,6 +376,7 @@ enum
 #ifdef HAVE_PGP
   OPTPGPAUTOSIGN,
   OPTPGPAUTOENCRYPT,
+  OPTPGPIGNORESUB,
   OPTPGPLONGIDS,
   OPTPGPREPLYENCRYPT,
   OPTPGPREPLYSIGN,
